﻿Public Class Client
    Implements IDisposable

    Public ClientConnected As Boolean
    Public WaveOut1 As WaveOut
    Private _Socket As Socket
    Private _ServerPort As Integer
    Private _ClientPort As Integer
    Private _Session As String = String.Empty
    Private _Host As String
    Private _WaveOutID As String
    Private _muLawToPcmMap As Short()
    Private _IPaddress As String = String.Empty
    Private _SeqCount As Integer = 1
    Private _PollTimer As System.Timers.Timer
    Private _ClientStopped As Boolean
    Private _UdpSocket As Socket
    Private _Buffer(-1) As Byte
    Private _TCPSScannerPort As Integer

    Public Event UpdateGUI(Code As Integer, Data As Object)

    Public Sub New(Host As String, WaveOutID As String, TCPSScannerPort As Integer)

        _Host = Host
        _WaveOutID = WaveOutID
        _TCPSScannerPort = TCPSScannerPort

    End Sub

    Public Sub Start()

        BytesTX = 0
        BytesRX = 0
        TotalPackets = 0
        LostPackets = 0
        StartTime = Nothing
        RaiseEvent UpdateGUI(4, New String() {BytesTX.ToString("#,##0"), BytesRX.ToString("#,##0"), FormatDuration(StartTime), "0.00" & " %"})

        Application.DoEvents()
        _muLawToPcmMap = New Short(255) {}
        For i As Byte = 0 To Byte.MaxValue - 1
            _muLawToPcmMap(i) = decode(i)
        Next
        SendInitHeaders()
        StartWaveOut()

        _PollTimer = New System.Timers.Timer(60000)
        AddHandler _PollTimer.Elapsed, AddressOf PollTimerEvent
        _PollTimer.Start()

        ThreadPool.QueueUserWorkItem(New WaitCallback(AddressOf SyncConnect))
        'AsyncConnect()

    End Sub

    Private Sub SyncConnect(State As Object)

        Dim bytesreceived As Integer
        Dim notfirsttime As Boolean

        Try
            _UdpSocket = New Socket(AddressFamily.InterNetwork, SocketType.Dgram, ProtocolType.Udp)
            _UdpSocket.Bind(New IPEndPoint(IPAddress.Any, _ClientPort)) 'LocalEP
            _UdpSocket.ReceiveTimeout = 4000
            ReDim _Buffer(_UdpSocket.ReceiveBufferSize - 1)

            While True
                If _ClientStopped = True Then Exit While
                Try
                    bytesreceived = _UdpSocket.Receive(_Buffer)
                Catch ex As Exception
                    ClientConnected = False
                    If _UdpSocket IsNot Nothing Then
                        _UdpSocket.Close()
                    End If
                    Return
                End Try

                If notfirsttime = False Then
                    StartTime = DateTime.Now
                    ClientConnected = True
                    notfirsttime = True
                    RaiseEvent UpdateGUI(6, Nothing) 'Cursors.Default
                End If
                BytesRX = BytesRX + bytesreceived

                ProcessData(_Buffer, bytesreceived)
            End While
        Catch ex As Exception
            ClientConnected = False
            If _UdpSocket IsNot Nothing Then
                _UdpSocket.Close()
            End If
            RaiseEvent UpdateGUI(3, "Unable To Connect" & vbCr & vbCr & ex.ToString)
            Return
        End Try

        ClientConnected = False
        If _UdpSocket IsNot Nothing Then
            _UdpSocket.Close()
        End If

    End Sub

    Private Sub AsyncConnect()

        Try
            _UdpSocket = New Socket(AddressFamily.InterNetwork, SocketType.Dgram, ProtocolType.Udp)
            _UdpSocket.Bind(New IPEndPoint(IPAddress.Any, _ClientPort)) 'LocalEP
            _UdpSocket.ReceiveTimeout = 4000
            ReDim _Buffer(_UdpSocket.ReceiveBufferSize - 1)
            StartTime = DateTime.Now
            ClientConnected = True
            RaiseEvent UpdateGUI(6, Nothing) 'Cursors.Default
            DoRead()
        Catch ex As Exception
            ClientConnected = False
            If _UdpSocket IsNot Nothing Then
                _UdpSocket.Close()
            End If
            RaiseEvent UpdateGUI(3, "Unable To Connect" & vbCr & vbCr & ex.ToString)
            Return
        End Try

    End Sub

    Private Sub DoRead()

        Try
            _UdpSocket.BeginReceive(_Buffer, 0, _Buffer.Length, SocketFlags.None, New AsyncCallback(AddressOf OnDataReceived), _UdpSocket)
        Catch ex As Exception
            ClientConnected = False
            If _UdpSocket IsNot Nothing Then
                _UdpSocket.Close()
            End If
            RaiseEvent UpdateGUI(3, "Unable To Connect" & vbCr & vbCr & ex.ToString)
            Return
        End Try

    End Sub

    Private Sub OnDataReceived(ar As IAsyncResult)

        Try
            If ClientConnected Then
                Dim bytesreceived As Integer = _UdpSocket.EndReceive(ar)
                If bytesreceived > 0 Then
                    BytesRX = BytesRX + bytesreceived

                    ProcessData(_Buffer, bytesreceived)

                    _UdpSocket.BeginReceive(_Buffer, 0, _Buffer.Length, SocketFlags.None, New AsyncCallback(AddressOf OnDataReceived), Nothing)
                Else
                    _UdpSocket.Close()
                End If
            End If
        Catch ex As Exception
            RaiseEvent UpdateGUI(3, "Unable To Connect" & vbCr & vbCr & ex.ToString)
            Return
        End Try

    End Sub

    Private Sub ProcessData(Buffer() As Byte, BytesReceived As Integer)
        'G711 30 ms
        '320 bytes of audio (332 total) each 40 ms - 8000 Samples / Second

        Dim buffer1 As [Byte]() = New [Byte](BytesReceived - 1) {}
        Array.Copy(Buffer, buffer1, BytesReceived)

        TotalPackets += 1
        Static tempSequenceNumber As Integer
        Dim SequenceNumber As UInt16 = System.BitConverter.ToUInt16(New Byte() {buffer1(3), buffer1(2)}, 0)
        If tempSequenceNumber <> 0 AndAlso SequenceNumber <> 0 AndAlso tempSequenceNumber + 1 <> SequenceNumber Then
            LostPackets += 1
            d("Packet Loss - " & SequenceNumber)
        End If
        tempSequenceNumber = SequenceNumber

        Dim buffer2(buffer1.Length - 13) As Byte
        Array.Copy(buffer1, 12, buffer2, 0, buffer1.Length - 12)

        If WaveOut1 IsNot Nothing Then
            If _ClientStopped = False Then
                WaveOut1.WaveOutPlay(MuLawDecode(buffer2))
            End If
        End If

    End Sub

    Private Sub SendInitHeaders()

        Dim sbHeader As New StringBuilder
        Dim bytesreceived As Integer
        Dim Bytes(-1) As Byte
        Dim bytessent() As Byte
        Dim str1 As String = String.Empty
        Dim data() As String
        Dim data1() As String
        Dim ScannerEndPoint As IPEndPoint

        Try
            data = _Host.Split(CChar("/"))
            If data.Length < 3 Then
                RaiseEvent UpdateGUI(3, "Unable To Open Connection" & vbCr & vbCr & "Invalid URL Format")
                Return
            End If
            _IPaddress = data(2)
            ScannerEndPoint = New IPEndPoint(System.Net.Dns.GetHostAddresses(_IPaddress)(0), _TCPSScannerPort)
            _Socket = New Socket(ScannerEndPoint.AddressFamily, SocketType.Stream, ProtocolType.Tcp)
            _Socket.ReceiveTimeout = 4000
            If _Socket Is Nothing Then
                RaiseEvent UpdateGUI(3, "Unable To Create Socket")
                Return
            End If
            _Socket.Connect(ScannerEndPoint)
            ReDim Bytes(_Socket.ReceiveBufferSize - 1)

            _SeqCount += 1
            sbHeader.Append("SETUP " & _Host & "/trackID=1 RTSP/1.0" & vbCrLf)
            sbHeader.Append("CSeq: " & _SeqCount & vbCrLf)
            sbHeader.Append("UserAgent: ProScan" & vbCrLf)
            sbHeader.Append("Transport: RTP/AVP;unicast;client_port=53600-53601" & vbCrLf)
            sbHeader.Append(vbCrLf)
            bytessent = Encoding.ASCII.GetBytes(sbHeader.ToString)
            _Socket.Send(bytessent)
            BytesTX = BytesTX + bytessent.Length
            bytesreceived = _Socket.Receive(Bytes, Bytes.Length, 0)
            BytesRX = BytesRX + bytesreceived
            str1 = Encoding.ASCII.GetString(Bytes, 0, bytesreceived)
            'd(str1)
            data = str1.Split(CChar(vbCrLf))
            For i As Integer = 0 To data.Length - 1
                data(i) = data(i).Replace(vbLf, String.Empty)
                If data(i).StartsWith("Session:") Then
                    _Session = data(i).Replace("Session:", String.Empty).Trim
                ElseIf data(i).StartsWith("Transport:") Then 'Transport: RTP/AVP;unicast;client_port=53600;source=192.168.1.5;server_port=56338;ssrc=15601863
                    data1 = data(i).Split(CChar(";"))
                    For j As Integer = 0 To data1.Length - 1
                        If data1(j).StartsWith("server_port=") Then
                            If IsNumeric(data1(j).Replace("server_port=", String.Empty)) Then _ServerPort = CInt(data1(j).Replace("server_port=", String.Empty))
                        ElseIf data1(j).StartsWith("client_port=") Then
                            If IsNumeric(data1(j).Replace("client_port=", String.Empty)) Then _ClientPort = CInt(data1(j).Replace("client_port=", String.Empty))
                        End If
                    Next
                End If
            Next
            RaiseEvent UpdateGUI(2, {_ServerPort.ToString, _ClientPort.ToString})

            _SeqCount += 1
            sbHeader.Length = 0
            sbHeader.Append("PLAY " & _Host & "/ RTSP/1.0" & vbCrLf)
            sbHeader.Append("CSeq: " & _SeqCount & vbCrLf)
            sbHeader.Append("UserAgent: ProScan" & vbCrLf)
            sbHeader.Append("Session: " & _Session & vbCrLf)
            sbHeader.Append("Range: npt=0.000-" & vbCrLf)
            sbHeader.Append(vbCrLf)
            bytessent = Encoding.ASCII.GetBytes(sbHeader.ToString)
            _Socket.Send(bytessent)
            BytesTX = BytesTX + bytessent.Length
            bytesreceived = _Socket.Receive(Bytes, Bytes.Length, 0)
            BytesRX = BytesRX + bytesreceived
        Catch ex As Exception
            RaiseEvent UpdateGUI(3, "Unable To Open Connection To " & _IPaddress & vbCr & vbCr & ex.ToString)
        End Try

    End Sub

    Private Sub PollTimerEvent(source As Object, e As ElapsedEventArgs)
        'keep alive poll

        Dim sbHeader As New StringBuilder
        Dim Bytes(511) As Byte
        Dim bytessent() As Byte
        Dim bytesreceived As Integer

        Try
            _SeqCount += 1
            sbHeader.Append("GET_PARAMETER " & _Host & "/ RTSP/1.0" & vbCrLf)
            sbHeader.Append("CSeq: " & _SeqCount & vbCrLf)
            sbHeader.Append("UserAgent: ProScan" & vbCrLf)
            sbHeader.Append("Session: " & _Session & vbCrLf)
            sbHeader.Append(vbCrLf)
            bytessent = Encoding.ASCII.GetBytes(sbHeader.ToString)
            _Socket.Send(bytessent)
            BytesTX = BytesTX + bytessent.Length
            bytesreceived = _Socket.Receive(Bytes, Bytes.Length, 0)
            'd(Encoding.ASCII.GetString(Bytes, 0, bytesreceived))
        Catch ex As Exception
        End Try

    End Sub

    Private Sub SendFinalHeaders()

        If _Socket Is Nothing Then Return

        Dim sbHeader As New StringBuilder
        Dim bytessent() As Byte
        Dim bytesreceived As Integer
        Dim Bytes(-1) As Byte

        Try
            _SeqCount += 1
            sbHeader.Append("TEARDOWN " & _Host & "/ RTSP/1.0" & vbCrLf)
            sbHeader.Append("CSeq: " & _SeqCount & vbCrLf)
            sbHeader.Append("UserAgent: ProScan" & vbCrLf)
            sbHeader.Append("Session: " & _Session & vbCrLf)
            sbHeader.Append(vbCrLf)
            bytessent = Encoding.ASCII.GetBytes(sbHeader.ToString)
            _Socket.Send(bytessent)
            BytesTX = BytesTX + bytessent.Length
            bytesreceived = _Socket.Receive(Bytes, Bytes.Length, 0)
            BytesRX = BytesRX + bytesreceived
        Catch ex As Exception
        End Try

    End Sub

    Public Sub StartWaveOut()

        If WaveOut1 IsNot Nothing Then
            WaveOut1.Dispose()
            WaveOut1 = Nothing
        End If

        If WaveOutID <> "None" Then
            If WaveOutID = String.Empty Then WaveOutID = "Primary Sound Playback Device"
            Dim DeviceID As Integer = GetWaveOutDeviceID(WaveOutID)
            If DeviceID = 9999 Then
                WaveOutID = "Primary Sound Playback Device"
                DeviceID = -1
            End If

            Dim WaveOutError As Boolean
            WaveOut1 = New WaveOut(DeviceID, "Mono", 8000, 50, WaveOutError)
            If WaveOutError = True Then
                If WaveOut1 IsNot Nothing Then
                    WaveOut1.Dispose()
                    WaveOut1 = Nothing
                End If
                RaiseEvent UpdateGUI(3, "Unable To Start Audio Playback")
                Return
            End If
            If WaveOut1 IsNot Nothing Then
                WaveOut1.Level = AudioLevel
                WaveOut1.Mute = AudioMute
            End If
        End If

    End Sub

    Private Function MuLawDecode(data As Byte()) As Short()

        Dim decoded As Short() = New Short(data.Length - 1) {}

        For i As Integer = 0 To data.Length - 1
            decoded(i) = _muLawToPcmMap(data(i))
        Next

        Return decoded

    End Function

    Private Function decode(mulaw As Byte) As Short

        mulaw = CByte(Not mulaw)

        Dim sign As Integer = mulaw And &H80
        Dim exponent As Integer = (mulaw And &H70) >> 4
        Dim data As Integer = mulaw And &HF

        data = data Or &H10
        data <<= 1
        data += 1
        data <<= exponent + 2
        data -= &H84

        Return CShort(If(sign = 0, data, -data))

    End Function

    Private Function ValueFromByte(value As Byte, startPos As Integer, length As Integer) As Integer

        Dim mask As Byte = 0

        For i As Integer = 0 To length - 1
            mask = CByte(mask Or &H1 << startPos + i)
        Next

        Dim result As Byte = (value And mask) >> startPos

        Return Convert.ToInt32(result)

    End Function

    Public Sub Close()

        _ClientStopped = True
        ClientConnected = False
        If WaveOut1 IsNot Nothing Then
            WaveOut1.ShutDown()
        End If
        SendFinalHeaders()
        If ProgramClosing = False Then
            If TotalPackets > 0 Then
                RaiseEvent UpdateGUI(4, New String() {BytesTX.ToString("#,##0"), BytesRX.ToString("#,##0"), FormatDuration(StartTime), CStr(Format(((LostPackets / TotalPackets) * 100), "0.00") & " %")})
            Else
                RaiseEvent UpdateGUI(4, New String() {BytesTX.ToString("#,##0"), BytesRX.ToString("#,##0"), FormatDuration(StartTime), "0.00" & " %"})
            End If
        End If
        If _Socket IsNot Nothing Then
            _Socket.Close()
        End If
        If _UdpSocket IsNot Nothing Then
            _UdpSocket.Close()
        End If
        Thread.Sleep(500)
        Dispose()

    End Sub

    Private disposedValue As Boolean = False        ' To detect redundant calls

    ' IDisposable
    Protected Overridable Sub Dispose(disposing As Boolean)

        If Not Me.disposedValue Then
            On Error Resume Next
            If _PollTimer IsNot Nothing Then
                _PollTimer.Close()
                RemoveHandler _PollTimer.Elapsed, AddressOf PollTimerEvent
                _PollTimer.Dispose()
                _PollTimer = Nothing
            End If
        End If
        Me.disposedValue = True

    End Sub

#Region " IDisposable Support "
    Public Sub Dispose() Implements IDisposable.Dispose
        ' Do not change this code.  Put cleanup code in Dispose(disposing As Boolean) above.
        Dispose(True)
        GC.SuppressFinalize(Me)
    End Sub
#End Region

End Class