﻿Public Class MessageBoxEx
    Private Shared _owner As IWin32Window
    Private Shared _hookProc As HookProc
    Private Shared _hHook As IntPtr
    Public Shared Function Show(owner As IWin32Window, text As String) As DialogResult
        _owner = owner
        Initialize()
        Return MessageBox.Show(owner, text)
    End Function
    Public Shared Function Show(owner As IWin32Window, text As String, caption As String) As DialogResult
        _owner = owner
        Initialize()
        Return MessageBox.Show(owner, text, caption)
    End Function
    Public Shared Function Show(owner As IWin32Window, text As String, caption As String, buttons As MessageBoxButtons) As DialogResult
        _owner = owner
        Initialize()
        Return MessageBox.Show(owner, text, caption, buttons)
    End Function
    Public Shared Function Show(owner As IWin32Window, text As String, caption As String, buttons As MessageBoxButtons, icon As MessageBoxIcon) As DialogResult
        _owner = owner
        Initialize()
        Return MessageBox.Show(owner, text, caption, buttons, icon)
    End Function
    Public Shared Function Show(owner As IWin32Window, text As String, caption As String, buttons As MessageBoxButtons, icon As MessageBoxIcon, defButton As MessageBoxDefaultButton) As DialogResult
        _owner = owner
        Initialize()
        Return MessageBox.Show(owner, text, caption, buttons, icon, defButton)
    End Function
    Public Shared Function Show(owner As IWin32Window, text As String, caption As String, buttons As MessageBoxButtons, icon As MessageBoxIcon, defButton As MessageBoxDefaultButton, options As MessageBoxOptions) As DialogResult
        _owner = owner
        Initialize()
        Return MessageBox.Show(owner, text, caption, buttons, icon, defButton, options)
    End Function
    Public Delegate Function HookProc(nCode As Integer, wParam As IntPtr, lParam As IntPtr) As IntPtr
    Public Delegate Sub TimerProc(hWnd As IntPtr, uMsg As UInteger, nIDEvent As UIntPtr, dwTime As UInteger)
    Public Const WH_CALLWNDPROCRET As Integer = 12
    Public Enum CbtHookAction As Integer
        HCBT_MOVESIZE = 0
        HCBT_MINMAX = 1
        HCBT_QS = 2
        HCBT_CREATEWND = 3
        HCBT_DESTROYWND = 4
        HCBT_ACTIVATE = 5
        HCBT_CLICKSKIPPED = 6
        HCBT_KEYSKIPPED = 7
        HCBT_SYSCOMMAND = 8
        HCBT_SETFOCUS = 9
    End Enum
    <DllImport("user32.dll")> _
    Private Shared Function GetWindowRect(hWnd As IntPtr, ByRef lpRect As Rectangle) As Boolean
    End Function
    <DllImport("user32.dll")> _
    Private Shared Function MoveWindow(hWnd As IntPtr, X As Integer, Y As Integer, nWidth As Integer, nHeight As Integer, bRepaint As Boolean) As Integer
    End Function
    <DllImport("User32.dll")> _
    Public Shared Function SetTimer(hWnd As IntPtr, nIDEvent As UIntPtr, uElapse As UInteger, lpTimerFunc As TimerProc) As UIntPtr
    End Function
    <DllImport("User32.dll")> _
    Public Shared Function SendMessage(hWnd As IntPtr, Msg As Integer, wParam As IntPtr, lParam As IntPtr) As IntPtr
    End Function
    <DllImport("user32.dll")> _
    Public Shared Function SetWindowsHookEx(idHook As Integer, lpfn As HookProc, hInstance As IntPtr, threadId As Integer) As IntPtr
    End Function
    <DllImport("user32.dll")> _
    Public Shared Function UnhookWindowsHookEx(idHook As IntPtr) As Integer
    End Function
    <DllImport("user32.dll")> _
    Public Shared Function CallNextHookEx(idHook As IntPtr, nCode As Integer, wParam As IntPtr, lParam As IntPtr) As IntPtr
    End Function
    <DllImport("user32.dll")> _
    Public Shared Function GetWindowTextLength(hWnd As IntPtr) As Integer
    End Function
    <DllImport("user32.dll")> _
    Public Shared Function GetWindowText(hWnd As IntPtr, text As StringBuilder, maxLength As Integer) As Integer
    End Function
    <DllImport("user32.dll")> _
    Public Shared Function EndDialog(hDlg As IntPtr, nResult As IntPtr) As Integer
    End Function
    <DllImport("kernel32.dll")> _
    Public Shared Function GetCurrentThreadId() As Integer
    End Function
    <StructLayout(LayoutKind.Sequential)> _
    Public Structure CWPRETSTRUCT
        Public lResult As IntPtr
        Public lParam As IntPtr
        Public wParam As IntPtr
        Public message As UInteger
        Public hwnd As IntPtr
    End Structure
    Shared Sub New()
        _hookProc = New HookProc(AddressOf MessageBoxHookProc)
        _hHook = IntPtr.Zero
    End Sub
    Private Shared Sub Initialize()
        If _owner IsNot Nothing Then
            _hHook = SetWindowsHookEx(WH_CALLWNDPROCRET, _hookProc, IntPtr.Zero, GetCurrentThreadId)
        End If
    End Sub
    Private Shared Function MessageBoxHookProc(nCode As Integer, wParam As IntPtr, lParam As IntPtr) As IntPtr
        If nCode < 0 Then
            Return CallNextHookEx(_hHook, nCode, wParam, lParam)
        End If
        Dim msg As CWPRETSTRUCT = CType(Marshal.PtrToStructure(lParam, GetType(CWPRETSTRUCT)), CWPRETSTRUCT)
        Dim hook As IntPtr = _hHook
        If msg.message = CInt(CbtHookAction.HCBT_ACTIVATE) Then
            Try
                CenterWindow(msg.hwnd)
            Finally
                UnhookWindowsHookEx(_hHook)
                _hHook = IntPtr.Zero
            End Try
        End If
        Return CallNextHookEx(hook, nCode, wParam, lParam)
    End Function
    Private Shared Sub CenterWindow(hChildWnd As IntPtr)
        Dim recChild As New Rectangle(0, 0, 0, 0)
        Dim success As Boolean = GetWindowRect(hChildWnd, recChild)
        Dim width As Integer = recChild.Width - recChild.X
        Dim height As Integer = recChild.Height - recChild.Y
        Dim recParent As New Rectangle(0, 0, 0, 0)
        success = GetWindowRect(_owner.Handle, recParent)
        Dim ptCenter As New Point(0, 0)
        ptCenter.X = recParent.X + ((recParent.Width - recParent.X) \ 2)
        ptCenter.Y = recParent.Y + ((recParent.Height - recParent.Y) \ 2)
        Dim ptStart As New Point(0, 0)
        ptStart.X = (ptCenter.X - (width \ 2))
        ptStart.Y = (ptCenter.Y - (height \ 2))
        ptStart.X = If((ptStart.X < 0), 0, ptStart.X)
        ptStart.Y = If((ptStart.Y < 0), 0, ptStart.Y)
        Dim rec As Rectangle = Screen.PrimaryScreen.WorkingArea()
        ptStart.X = If((ptStart.X > rec.Width - width), rec.Width - width, ptStart.X)
        ptStart.Y = If((ptStart.Y > rec.Height - height), rec.Height - height, ptStart.Y)
        Dim result As Integer = MoveWindow(hChildWnd, ptStart.X, ptStart.Y, width, height, False)
    End Sub
End Class
