﻿Imports System.Drawing.Drawing2D

Public Class VolumeControl

    Private _Value1 As Integer
    Private _MouseDown As Boolean
    Private _GraphicsPath As GraphicsPath
    Private _Font As Font
    Private _Mute As Boolean

    Public Event ValueChanged(Value As Integer)
    Public Event MuteChanged(Value As Boolean)

    Private Sub VolumeControl_Load(sender As Object, e As System.EventArgs) Handles Me.Load

        SetStyle(ControlStyles.OptimizedDoubleBuffer, True)

        _Font = New Font("Microsoft Sans Serif", 9.0!, FontStyle.Regular, GraphicsUnit.Point, CType(0, Byte))

        _GraphicsPath = New GraphicsPath
        _GraphicsPath.AddArc(Me.Width - 7, 1 + 8, 6, 6, 250, 130)
        _GraphicsPath.AddLine(Me.Width - 1, 6 + 8, Me.Width - 1, Me.Height - 7) '|
        _GraphicsPath.AddArc(Me.Width - 9, Me.Height - 9, 8, 8, 340, 130)
        _GraphicsPath.AddLine(4, Me.Height - 1, Me.Width - 7, Me.Height - 1) '-
        _GraphicsPath.AddArc(0, Me.Height - 5, 4, 4, 80, 180)
        _GraphicsPath.AddLine(3, Me.Height - 5, Me.Width - 6, 1 + 8) '/
        _GraphicsPath.CloseAllFigures()

        PictureBox1.Image = My.Resources.MuteOff16

    End Sub

    Private Sub VolumeControl_Resize(sender As Object, e As System.EventArgs) Handles Me.Resize

        Me.Width = 102 'keep at 102 (0,1 - 99, + width always 1 less
     
    End Sub

    Private Sub VolumeControl_Disposed(sender As Object, e As System.EventArgs) Handles Me.Disposed

        If _Font IsNot Nothing Then _Font.Dispose()
        If _GraphicsPath IsNot Nothing Then _GraphicsPath.Dispose()

    End Sub

    Private Sub VolumeControl_MouseDown(sender As Object, e As System.Windows.Forms.MouseEventArgs) Handles Me.MouseDown

        If _GraphicsPath.IsVisible(e.X, e.Y) = True Then
            _MouseDown = True
        End If

    End Sub

    Private Sub VolumeControl_MouseUp(sender As Object, e As System.Windows.Forms.MouseEventArgs) Handles Me.MouseUp

        If _GraphicsPath.IsVisible(e.X, e.Y) = True OrElse _MouseDown = True Then
            _MouseDown = False
        End If

    End Sub

    Private Sub VolumeControl_MouseMove(sender As Object, e As System.Windows.Forms.MouseEventArgs) Handles Me.MouseMove

        If _GraphicsPath.IsVisible(e.X, e.Y) = True AndAlso _MouseDown = True Then
            _Value1 = e.X
            CommonValueChange(True, True)
        End If

    End Sub

    Private Sub VolumeControl_Paint(sender As Object, e As System.Windows.Forms.PaintEventArgs) Handles Me.Paint

        e.Graphics.SmoothingMode = Drawing2D.SmoothingMode.AntiAlias
        e.Graphics.FillPath(Brushes.WhiteSmoke, _GraphicsPath)
        e.Graphics.DrawPath(Pens.Gray, _GraphicsPath)
        e.Graphics.DrawString(_Value1.ToString & "%", _Font, Brushes.DimGray, New Point(26, -2))
        e.Graphics.Clip = New Region(New Rectangle(0, 0, _Value1 + 2, Me.Height))
        e.Graphics.FillPath(Brushes.LimeGreen, _GraphicsPath)

    End Sub

    Protected Overrides Function IsInputKey(key As Keys) As Boolean

        Select Case key
            Case Keys.Up, Keys.Right
                _Value1 += 2
                CommonValueChange(True, False)
                Return True
            Case Keys.Down, Keys.Left
                _Value1 -= 2
                CommonValueChange(True, False)
                Return True
            Case Else
                Return False
        End Select

        Return MyBase.IsInputKey(key)

    End Function

    Private Sub CommonValueChange(ChangeMute As Boolean, FromMouse As Boolean)

        Dim tempmute As Boolean = _Mute

        If _Value1 < If(FromMouse, 5, 0) Then
            _Value1 = 0
            _Mute = True
        ElseIf _Value1 > If(FromMouse, 95, 100) Then
            _Value1 = 100
            _Mute = False
        Else
            _Mute = False
        End If

        Static tempLevel As Integer
        If tempLevel <> _Value1 Then
            Invalidate()
            RaiseEvent ValueChanged(_Value1)
        End If
        tempLevel = _Value1

        If tempmute <> _Mute Then
            If ChangeMute = True Then CommonMuteChange(True)
        End If

    End Sub

    Private Sub CommonMuteChange([RaiseEvent] As Boolean)

        If _Mute = False Then
            PictureBox1.Image = My.Resources.MuteOff16
            If [RaiseEvent] = True Then RaiseEvent MuteChanged(False)
        Else
            PictureBox1.Image = My.Resources.MuteOn16
            If [RaiseEvent] = True Then RaiseEvent MuteChanged(True)
        End If
        
    End Sub

    Private Sub PictureBox1_Click(sender As Object, e As System.EventArgs) Handles PictureBox1.Click

        _Mute = Not _Mute
        CommonMuteChange(True)

    End Sub

    Private Sub PictureBox1_MouseHover(sender As Object, e As System.EventArgs) Handles PictureBox1.MouseHover

        PictureBox1.BorderStyle = Windows.Forms.BorderStyle.FixedSingle

    End Sub

    Private Sub PictureBox1_MouseLeave(sender As Object, e As System.EventArgs) Handles PictureBox1.MouseLeave

        PictureBox1.BorderStyle = Windows.Forms.BorderStyle.None

    End Sub

    Public Property Level As Integer

        Get
            Level = _Value1
        End Get

        Set(value As Integer)
            _Value1 = value
            CommonValueChange(False, False)
        End Set

    End Property

    Public Property Mute As Boolean

        Get
            Mute = _Mute
        End Get

        Set(value As Boolean)
            _Mute = value
            CommonMuteChange(False)
        End Set

    End Property

End Class
