﻿Imports System.Runtime.InteropServices

Public Class WaveOut
    Implements IDisposable

    Private Declare Function waveOutOpen Lib "winmm.dll" (ByRef WaveOutPtr As IntPtr, ByVal uDeviceID As Integer, ByRef lpFormat As Wave.WAVEFORMATEX, ByVal dwCallback As IntPtr, ByVal dwInstance As Integer, ByVal dwFlags As Integer) As Integer
    Private Declare Function waveOutWrite Lib "winmm.dll" (ByVal WaveOutPtr As IntPtr, ByRef lpWaveOutHdr As Wave.WAVEHDR, ByVal uSize As Integer) As Integer
    Private Declare Function waveOutPrepareHeader Lib "winmm.dll" (ByVal WaveOutPtr As IntPtr, ByRef lpWaveOutHdr As Wave.WAVEHDR, ByVal uSize As Integer) As Integer
    Private Declare Function waveOutUnprepareHeader Lib "winmm.dll" (ByVal WaveOutPtr As IntPtr, ByRef lpWaveOutHdr As Wave.WAVEHDR, ByVal uSize As Integer) As Integer
    Private Declare Function waveOutReset Lib "winmm.dll" (ByVal WaveOutPtr As IntPtr) As Integer
    Private Declare Function waveOutClose Lib "winmm.dll" (ByVal WaveOutPtr As IntPtr) As Integer

    Private _WaveFormat As Wave.WAVEFORMATEX
    Private _WaveHeader(-1) As Wave.WAVEHDR
    Private _HeaderHandle As GCHandle
    Private _WavePtr As IntPtr
    Private _Result As Integer
    Private _CountBuffer As Integer
    Private _DeviceID As Integer
    Private _SamplesPerSec As Integer
    Private _Buffers As Integer
    Private _ShutDown As Boolean
    Private _Level As Integer
    Private _Mute As Boolean

    Public Sub New(ByVal DeviceID As Integer, ByVal ChannelMode As String, ByVal SamplesPerSec As Integer, ByVal Buffers As Integer, ByRef WaveError As Boolean)

        _DeviceID = DeviceID
        _SamplesPerSec = SamplesPerSec
        _Buffers = Buffers
        ReDim _WaveHeader(_Buffers - 1)

        With _WaveFormat
            .wFormatTag = Wave.WAVE_FORMAT_PCM
            .nChannels = CShort(IIf(ChannelMode = "Stereo", 2, 1))
            .nSamplesPerSec = _SamplesPerSec
            .wBitsPerSample = 16
            .nBlockAlign = CShort(.nChannels * (.wBitsPerSample / 8))
            .nAvgBytesPerSec = .nSamplesPerSec * .nBlockAlign
        End With

        _Result = waveOutOpen(_WavePtr, _DeviceID, _WaveFormat, IntPtr.Zero, 0, Wave.CALLBACK_NULL)
        If _Result <> Wave.MMSYSERR_NOERROR Then
            WaveError = True
            Return
        End If

        _HeaderHandle = GCHandle.Alloc(_WaveHeader, GCHandleType.Pinned)
        For Me._CountBuffer = 0 To _Buffers - 1
            With _WaveHeader(Me._CountBuffer)
                .lpData = Marshal.AllocHGlobal(CInt(_WaveFormat.nAvgBytesPerSec * 2))
                .dwUser = New IntPtr(Me._CountBuffer)
                .dwBufferLength = CInt(_WaveFormat.nAvgBytesPerSec * 2)
                .dwBytesRecorded = 0
                .dwFlags = 0
                .dwLoops = 0
                .lpNext = IntPtr.Zero
                .reserved = 0
            End With
            _Result = waveOutPrepareHeader(_WavePtr, _WaveHeader(Me._CountBuffer), Marshal.SizeOf(_WaveHeader(Me._CountBuffer)))
            If _Result <> Wave.MMSYSERR_NOERROR Then
                WaveError = True
                Return
            End If
        Next

        Me._CountBuffer = 0

    End Sub

    Public Sub WaveOutPlay(ByVal Buffer() As Short)

        If _WaveHeader Is Nothing Then Return
        If _WavePtr = IntPtr.Zero Then Return
        If Me._CountBuffer = _Buffers Then Me._CountBuffer = 0
        If _WaveHeader(Me._CountBuffer).lpData = IntPtr.Zero Then Return
        If _ShutDown = True Then Return

        For i = 0 To Buffer.Length - 1
            If _Mute = True Then
                Buffer(i) = 0
            Else
                Buffer(i) = CShort(Buffer(i) * (_Level / 100))
            End If
        Next

        Try
            Marshal.Copy(Buffer, 0, _WaveHeader(Me._CountBuffer).lpData, Buffer.Length)
            _WaveHeader(Me._CountBuffer).dwBufferLength = Buffer.Length * 2
            If _ShutDown = True Then Return
            'don't use flags because Win ME will fail
            _Result = waveOutWrite(_WavePtr, _WaveHeader(Me._CountBuffer), Marshal.SizeOf(_WaveHeader(Me._CountBuffer)))
            Me._CountBuffer += 1
        Catch ex As Exception
        End Try

    End Sub

    Public Sub ShutDown()

        _ShutDown = True

    End Sub

    Public WriteOnly Property Level As Integer

        Set(value As Integer)
            _Level = value
        End Set

    End Property

    Public WriteOnly Property Mute As Boolean

        Set(value As Boolean)
            _Mute = value
        End Set

    End Property

    Private disposedValue As Boolean = False 'detects redundant calls

    ' IDisposable
    Protected Overridable Sub Dispose(ByVal disposing As Boolean)
        If Not Me.disposedValue Then
            On Error Resume Next
            Me.disposedValue = True 'keep in this position
            waveOutReset(_WavePtr)
            If _HeaderHandle.IsAllocated Then _HeaderHandle.Free()
            For Me._CountBuffer = 0 To _Buffers - 1
                waveOutUnprepareHeader(_WavePtr, _WaveHeader(Me._CountBuffer), Marshal.SizeOf(_WaveHeader(Me._CountBuffer)))
                Marshal.FreeHGlobal(_WaveHeader(Me._CountBuffer).lpData)
            Next
            waveOutClose(_WavePtr)
            _WavePtr = IntPtr.Zero
            _WavePtr = Nothing
            _WaveHeader = Nothing
        End If
    End Sub

#Region " IDisposable Support "
    ' This code added by Visual Basic to correctly implement the disposable pattern.
    Public Sub Dispose() Implements IDisposable.Dispose
        ' Do not change this code.  Put cleanup code in Dispose(ByVal disposing As Boolean) above.
        Dispose(True)
        GC.SuppressFinalize(Me)
    End Sub
#End Region

End Class

Public Class Wave

    Public Const MMSYSERR_NOERROR As Integer = 0
    Public Const WAVE_MAPPER As Integer = -1
    Public Const WAVE_FORMAT_PCM As Integer = 1
    Public Const WHDR_DONE As Integer = &H1
    Public Const WOM_DONE As Integer = &H3BD
    Public Const CALLBACK_NULL As Integer = &H0
    Public Const CALLBACK_WINDOW As Integer = &H10000
    Public Const CALLBACK_THREAD As Integer = &H20000
    Public Const CALLBACK_FUNCTION As Integer = &H30000
    Public Const CALLBACK_EVENT As Integer = &H50000

    Public Declare Function waveInGetNumDevs Lib "winmm.dll" () As Integer
    Public Declare Function waveInGetDevCaps Lib "winmm.dll" Alias "waveInGetDevCapsA" (ByVal uDeviceID As Integer, ByRef lpCaps As WAVEINCAPS, ByVal uSize As Integer) As Integer
    Public Declare Function waveOutGetNumDevs Lib "winmm.dll" () As Integer
    Public Declare Function waveOutGetDevCaps Lib "winmm.dll" Alias "waveOutGetDevCapsA" (ByVal uDeviceID As Integer, ByRef lpCaps As WAVEOUTCAPS, ByVal uSize As Integer) As Integer

    <StructLayoutAttribute(LayoutKind.Sequential)> _
    Public Structure WAVEHDR
        Dim lpData As IntPtr
        Dim dwBufferLength As Integer
        Dim dwBytesRecorded As Integer
        Dim dwUser As IntPtr
        Dim dwFlags As Integer
        Dim dwLoops As Integer
        Dim lpNext As IntPtr
        Dim reserved As Integer
    End Structure

    <StructLayoutAttribute(LayoutKind.Sequential)> _
    Public Structure WAVEFORMATEX
        Dim wFormatTag As Short
        Dim nChannels As Short
        Dim nSamplesPerSec As Integer
        Dim nAvgBytesPerSec As Integer
        Dim nBlockAlign As Short
        Dim wBitsPerSample As Short
    End Structure

    <StructLayout(LayoutKind.Sequential)> _
    Public Structure WAVEINCAPS
        Dim wMid As Short
        Dim wPid As Short
        Dim vDriverVersion As Integer
        <MarshalAs(UnmanagedType.ByValTStr, SizeConst:=32)> _
        Dim szPname As String
        Dim dwFormats As Integer
        Dim wChannels As Short
        Dim wReserved As Short
    End Structure

    <StructLayout(LayoutKind.Sequential)> _
    Public Structure WAVEOUTCAPS
        Dim wMid As Short
        Dim wPid As Short
        Dim vDriverVersion As Integer
        <MarshalAs(UnmanagedType.ByValTStr, SizeConst:=32)> _
        Dim szPname As String
        Dim dwFormats As Integer
        Dim wChannels As Short
        Dim dwSupport As Integer
    End Structure

End Class
